# load required packages, potentially install them. the install commands
# are commented out
# --------------------------------------------------------------------------------
#install.packages("INLA",repos=c(getOption("repos"), INLA="https://inla.r-inla-download.org/R/testing"), dep=TRUE)
library(INLA)
#install.packages("BiocManager")
library(BiocManager)
#BiocManager::install("minfiData")
BiocManager::install("minfi")
library(minfiData)
library(minfi)
#BiocManager::install("RnBeads")
#BiocManager::install("RnBeads.hg19")
#BiocManager::install("wateRmelon")
#BiocManager::install("RPMM")
library(RnBeads)
library(RnBeads.hg19)
library(wateRmelon)
library(RPMM)
# --------------------------------------------------------------------------------

# load example data:
baseDir <- system.file("extdata", package="minfiData")
targets <- read.metharray.sheet(baseDir)
RGSet <- read.metharray.exp(targets = targets)
phenoData <- pData(RGSet)
MSet <- preprocessRaw(RGSet)  # choose your preprocessing method; Noob, Raw, Quantile, etc.
ratioSet = ratioConvert(MSet, what = "both", keepCN = TRUE)
betas = getBeta(ratioSet)

# remove sites with bead counts less than 3, p-value higher than 0.01 and exactly 0 and 1:
pValues = detectionP(rgSet = RGSet)
CNset = getCN(ratioSet)
for(i in 1:dim(betas)[2]){
  betas[pValues[, i] > 0.01, i] = NA
  betas[CNset[, i] < 3, i] = NA  # often overlap with those set to NA based on pValue

}
# in this example, only 6 samples, so lets remove all CpGs with one or 
# more missing value (instead of the CpGs with more than 5% samples with missing value):
betas[betas == 0] = NA
betas[betas == 1] = NA
betas = na.omit(betas)

# import and create relevant genomic information for probes:
probes450All <- rnb.get.annotation("probes450")

ChromeNamesParted = list() # when done, list of 22 elements where each 
# element is a vetor with the IDs of cg-sites along chrome. i (1-22)

ChromeLocationsParted = list() # when done, list of 22 elements where each 
# element is a vector with the start location of the corresponding 
# cg-site in ChromeNamesParted

probe_type = list() # probe type design for each probe, I or II

chrVec = c("chr1", "chr2","chr3", "chr4","chr5", "chr6","chr7",
           "chr8","chr9", "chr10","chr11", "chr12","chr13", "chr14",
           "chr15", "chr16","chr17", "chr18","chr19", "chr20","chr21", "chr22", "chrX", "chrY" )
count = 1
for(i in chrVec){
  ChromeLocationsParted[[count]] = data.frame(ranges(probes450All[[i]]))[,1]
  ChromeNamesParted[[count]] = data.frame(ranges(probes450All[[i]]))[,4]
  names(ChromeLocationsParted[[count]]) = ChromeNamesParted[[count]]
  probe_type[[count]] = as.numeric(probes450All[[i]]$Design)
  names(probe_type[[count]]) = ChromeNamesParted[[count]]
  count = count + 1  
}

# remove sex chromosomes:
sex_chrome_probes = c(ChromeNamesParted[[23]], ChromeNamesParted[[24]])
betas = betas[setdiff(rownames(betas), sex_chrome_probes), ]

# BMIQ:
collapse_probe_type = c()
for(i in 1:length(probe_type)){
  collapse_probe_type = c(collapse_probe_type, probe_type[[i]])
}
collapse_probe_type = collapse_probe_type[rownames(betas)]  # vector of probe types

betas_bmiq = betas
for(i in 1:dim(betas)[2]){
  betas_bmiq[, i] = BMIQ(betas[, i], collapse_probe_type)$nbeta  
}

# transform to M values:
M_bmiq = logit2(betas_bmiq)

# normalize on population mean and variance:
M_bmiq_norm = (M_bmiq - rowMeans(M_bmiq))/rowSds(M_bmiq)

# INLA fitting:
sigma0 = 1
range0 = 3000
alpha = 1
d = 1
lambda = alpha - d/2
kappa0 = sqrt(8*lambda)/range0 
tau0 = sqrt(gamma(lambda)/(gamma(alpha)*(4*pi)^(d/2)*kappa0^(2*lambda)))/sigma0 

SomeChromes = c(5)  # chose one or more chromosomes to fit dependency structure for, 
# chromosome 5 chosen here
SaveOutPut = list()
count = 1
for(j in 1:dim(M_bmiq_norm)[2]){  # 1:6
  for(i in SomeChromes){    # 1:22
    ChromeNames = intersect(ChromeNamesParted[[i]], names(M_bmiq_norm[, j])) 
    n = length(ChromeNames)
    Locations = ChromeLocationsParted[[i]][ChromeNames]  # locations
    LocS = inla.mesh.1d(loc = Locations)  # mesh
    spdeStat = inla.spde2.matern(mesh = LocS, alpha = alpha, B.tau = matrix(c(0,1,0), nrow = 1, ncol = 3), 
                                 B.kappa = matrix(c(0,0,1), nrow = 1, ncol = 3), 
                                 theta.prior.mean = c(log(tau0), log(kappa0)), theta.prior.prec = c(0.05,0.05)) # define the covariance 
                                              #  function, and priors for tau and kappa
    
    IndexS = inla.spde.make.index(name = "spatial.field", n.spde = n, n.repl = 1)  # make index for the fitting
    intercept = rep(1, times = n)
    formulaStat = y ~ -1 + intercept + f(spatial.field, model = spdeStat, n = n)
    y = M_bmiq_norm[ChromeNames, j]  # y is the data to be fitted
    
    spdeOutStat = inla(formulaStat, data = list(y = y, spatial.field = IndexS$spatial.field, 
                                                spdeStat = spdeStat, intercept = intercept), 
                       family = "gaussian", control.compute = list(dic = TRUE), 
                       control.inla = list(reordering="metis")) # inla execution
    
    SaveOutPut[[count]] = spdeOutStat  # store everything in a list
    print(j)
    print(i)
    count = count + 1
  }
}

# See https://www.r-inla.org/ for information about structure for output

# posterior marginal distributions for log(tau)
plot(SaveOutPut[[1]]$marginals.hyperpar$`Theta1 for spatial.field`, type = "l", xlim = c(2, 6))
points(SaveOutPut[[2]]$marginals.hyperpar$`Theta1 for spatial.field`, type = "l", col = "red")
points(SaveOutPut[[3]]$marginals.hyperpar$`Theta1 for spatial.field`, type = "l", col = "orange")
points(SaveOutPut[[4]]$marginals.hyperpar$`Theta1 for spatial.field`, type = "l", col = "blue")
points(SaveOutPut[[5]]$marginals.hyperpar$`Theta1 for spatial.field`, type = "l", col = "darkgreen")
points(SaveOutPut[[6]]$marginals.hyperpar$`Theta1 for spatial.field`, type = "l", col = "yellow")

#posterior marginals distributions for log(kappa)
plot(SaveOutPut[[1]]$marginals.hyperpar$`Theta2 for spatial.field`, type = "l", xlim = c(-10, -5))
points(SaveOutPut[[2]]$marginals.hyperpar$`Theta2 for spatial.field`, type = "l", col = "red")
points(SaveOutPut[[3]]$marginals.hyperpar$`Theta2 for spatial.field`, type = "l", col = "orange")
points(SaveOutPut[[4]]$marginals.hyperpar$`Theta2 for spatial.field`, type = "l", col = "blue")
points(SaveOutPut[[5]]$marginals.hyperpar$`Theta2 for spatial.field`, type = "l", col = "darkgreen")
points(SaveOutPut[[6]]$marginals.hyperpar$`Theta2 for spatial.field`, type = "l", col = "yellow")


